package cz.muni.fi.pa165.core.user;

import cz.muni.fi.pa165.model.dto.common.Result;
import cz.muni.fi.pa165.model.dto.user.UserCreateDto;
import cz.muni.fi.pa165.model.dto.user.UserDto;
import cz.muni.fi.pa165.model.dto.user.UserStatisticsCreateDto;
import cz.muni.fi.pa165.model.dto.user.UserUpdateDto;
import jakarta.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/api/user")
@Tag(name = "user", description = "User API")
public class UserController {
	private final UserFacade userFacade;
	private final UserService userService; // TODO: this could be done in UserFacade

	@Autowired
	public UserController(UserFacade userFacade, UserService userService) {
		this.userFacade = userFacade;
		this.userService = userService;
	}

  @Operation(
      summary = "Find user by ID",
      description = "Returns a single user",
      tags = {"user"})
  @ApiResponses(
      value = {
        @ApiResponse(
            responseCode = "200",
            description = "User found",
            content = {
              @Content(
                  mediaType = "application/json",
                  schema = @Schema(implementation = UserDto.class))
            }),
        @ApiResponse(responseCode = "404", description = "User not found", content = @Content)
      })
  @GetMapping("/{id}")
  public UserDto findById(
      @Parameter(description = "ID of user to be searched") @PathVariable String id) {
    return userFacade.findById(id);
  }

  @Operation(
      summary = "Create user",
      description = "Creates a new user",
      tags = {"user"})
  @ApiResponses(
      value = {
        @ApiResponse(
            responseCode = "201",
            description = "User created",
            content = {
              @Content(
                  mediaType = "application/json",
                  schema = @Schema(implementation = UserDto.class))
            }),
        @ApiResponse(responseCode = "400", description = "Invalid input", content = @Content),
        @ApiResponse(
            responseCode = "409",
            description = "User with the same name already exists",
            content = @Content)
      })
  @PostMapping
  public UserDto create(
      @Parameter(description = "User to be created") @RequestBody UserCreateDto userCreateDto) {
    return userFacade.create(userCreateDto);
  }

  @Operation(
      summary = "Get all users",
      description = "Returns all users",
      tags = {"user"})
  @ApiResponses(
      value = {
        @ApiResponse(
            responseCode = "200",
            description = "Users found",
            content = {
              @Content(
                  mediaType = "application/json",
                  schema = @Schema(implementation = Result.class))
            })
      })
  @GetMapping
  public Result<UserDto> findAll(
      @Parameter(description = "Page number of results to retrieve") @RequestParam int page) {
    return userFacade.findAll(page);
  }

	@PutMapping("/{id}")
	@Operation(summary = "Update a user by ID")
	@ApiResponses(value = {
			@ApiResponse(responseCode = "200", description = "User updated successfully"),
			@ApiResponse(responseCode = "400", description = "Invalid input"),
			@ApiResponse(responseCode = "404", description = "User not found"),
			@ApiResponse(responseCode = "500", description = "Internal server error")
	})
	public UserDto updateById(
			@Parameter(description = "ID of the user to be updated") @PathVariable String id,
			@RequestBody @Valid UserUpdateDto userUpdateDto) {
		return userFacade.updateById(userUpdateDto, id);
	}

	@DeleteMapping("/{id}")
	@Operation(summary = "Delete a user by ID")
	@ApiResponses(value = {
			@ApiResponse(responseCode = "200", description = "User deleted successfully"),
			@ApiResponse(responseCode = "404", description = "User not found"),
			@ApiResponse(responseCode = "500", description = "Internal server error")
	})
	public UserDto deleteById(
			@Parameter(description = "ID of the user to be deleted") @PathVariable String id) {
		return userFacade.deleteById(id);
	}

	@PostMapping("/statistics")
	@Operation(summary = "Send a user house with data")
	@ApiResponses(value = {
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(responseCode = "404", description = "User not found"),
	})
	public List<Double> userHouseStatistics(
			@RequestBody UserStatisticsCreateDto createStatDto) {
		return userService.getConsumption(createStatDto.getId(), createStatDto.getHouseId(),
				createStatDto.getStartTime(), createStatDto.getEndTime());
	}

	@Operation(
			summary = "Get all users",
			description = "Returns all users",
			tags = {"user"})
	@ApiResponses(
			value = {
					@ApiResponse(
							responseCode = "200",
							description = "Users found",
							content = {
									@Content(
											mediaType = "application/json",
											schema = @Schema(implementation = Result.class))
							})
			})
	@GetMapping("/all")
	public List<UserDto> findAll() {
		return userFacade.findAll();
	}

	/*
		TODO: get user with roles
	 */
}
